@@ -9,8 +9,8 @@
 ;;             Bozhidar Batsov <bozhidar@batsov.dev>
 ;; URL: https://github.com/copilot-emacs/copilot.el
 ;; Package-Requires: ((emacs "27.2") (editorconfig "0.8.2") (jsonrpc "1.0.14") (compat "30") (track-changes "1.4"))
-;; Package-Version: 20260219.2146
-;; Package-Revision: a62c1a573371
+;; Package-Version: 20260225.538
+;; Package-Revision: 0e1c017162f5
 ;; Keywords: convenience copilot
 
 ;; The MIT License (MIT)
@@ -475,12 +475,14 @@ recently updated session."
   (and copilot--connection
        (zerop (process-exit-status (jsonrpc--process copilot--connection)))))
 
-(defmacro copilot--request (&rest args)
-  "Send a request to the copilot server with ARGS."
+(defmacro copilot--request (method &optional params &rest args)
+  "Send a request to the copilot server for METHOD with PARAMS and ARGS.
+When PARAMS is nil, send an empty JSON object so the server does not
+reject the request with a schema-validation error."
   `(progn
      (unless (copilot--connection-alivep)
        (copilot--start-server))
-     (jsonrpc-request copilot--connection ,@args)))
+     (jsonrpc-request copilot--connection ,method (or ,params (make-hash-table)) ,@args)))
 
 (defmacro copilot--notify (&rest args)
   "Send a notification to the copilot server with ARGS."
@@ -1291,15 +1293,13 @@ provided."
         (copilot-clear-overlay t)
         (if (derived-mode-p 'vterm-mode)
             (progn
-              (vterm-delete-region start end)
+              (unless is-partial (vterm-delete-region start end))
               (vterm-insert t-completion))
-          (delete-region start end)
+          (unless is-partial (delete-region start end))
           (insert t-completion))
         ;; if it is a partial completion, show remaining text
         (when is-partial
-          (let ((ov (copilot--get-overlay)))
-            (overlay-put ov 'tail-length (- (line-end-position) (point)))
-            (copilot--set-overlay-text ov (string-remove-prefix t-completion completion)))))
+          (copilot--set-overlay-text (copilot--get-overlay) (string-remove-prefix t-completion completion))))
       t)))
 
 (defmacro copilot--define-accept-completion-by-action (func-name action)
@@ -1397,6 +1397,28 @@ Uppercase CHAR disables `case-fold-search', mirroring `zap-to-char'."
             (pcase-let ((`(,start ,end ,balanced-text) fixed-completion))
               (copilot--display-overlay-completion balanced-text command full-insert-text start end))))))))
 
+(defun copilot--ensure-doc-open ()
+  "Ensure the current buffer has been opened with the Copilot server.
+Sends workspace folder and `textDocument/didOpen' notifications if
+the buffer has not been registered yet.  Safe to call multiple times."
+  (when-let* ((root (copilot--workspace-root))
+              (root-uri (copilot--path-to-uri root)))
+    (unless (member root-uri copilot--workspace-folders)
+      (push root-uri copilot--workspace-folders)
+      (copilot--notify 'workspace/didChangeWorkspaceFolders
+                       (list :event
+                             (list :added (vector (list :uri root-uri
+                                                        :name (file-name-nondirectory
+                                                               (directory-file-name root))))
+                                   :removed [])))))
+  (unless (seq-contains-p copilot--opened-buffers (current-buffer))
+    (add-to-list 'copilot--opened-buffers (current-buffer))
+    (copilot--notify 'textDocument/didOpen
+                     (list :textDocument (list :uri (copilot--get-uri)
+                                               :languageId (copilot--get-language-id)
+                                               :version copilot--doc-version
+                                               :text (copilot--get-source))))))
+
 (defun copilot--on-doc-focus (window)
   "Notify that the document WINDOW has been focussed or opened."
   ;; When switching windows, this function is called twice, once for the
@@ -1404,26 +1426,10 @@ Uppercase CHAR disables `case-fold-search', mirroring `zap-to-char'."
   ;; send a notification for the window gaining focus and only if the buffer has
   ;; copilot-mode enabled.
   (when (and copilot-mode (eq window (selected-window)))
-    ;; Detect new workspace roots and notify the server.
-    (when-let* ((root (copilot--workspace-root))
-                (root-uri (copilot--path-to-uri root)))
-      (unless (member root-uri copilot--workspace-folders)
-        (push root-uri copilot--workspace-folders)
-        (copilot--notify 'workspace/didChangeWorkspaceFolders
-                         (list :event
-                               (list :added (vector (list :uri root-uri
-                                                          :name (file-name-nondirectory
-                                                                 (directory-file-name root))))
-                                     :removed [])))))
     (if (seq-contains-p copilot--opened-buffers (current-buffer))
         (copilot--notify 'textDocument/didFocus
                          (list :textDocument (list :uri (copilot--get-uri))))
-      (add-to-list 'copilot--opened-buffers (current-buffer))
-      (copilot--notify 'textDocument/didOpen
-                       (list :textDocument (list :uri (copilot--get-uri)
-                                                 :languageId (copilot--get-language-id)
-                                                 :version copilot--doc-version
-                                                 :text (copilot--get-source)))))))
+      (copilot--ensure-doc-open))))
 
 (defun copilot--on-doc-close (&rest _args)
   "Notify that the document has been closed."
@@ -1437,6 +1443,7 @@ Uppercase CHAR disables `case-fold-search', mirroring `zap-to-char'."
 (defun copilot-complete ()
   "Complete at the current point."
   (interactive)
+  (copilot--ensure-doc-open)
   (setq copilot--last-doc-version copilot--doc-version)
 
   (setq copilot--completion-cache nil)
